# rubocop:disable Betterment/AllowlistBlocklist
module RuboCop
  module Cop
    module Betterment
      class AllowlistBlocklist < Cop
        MSG = <<-DOC.freeze
          Betterment has moved away from usages of whitelist & blacklist in favor of more inclusive terms.
          Replace any instances of these terms with more inclusive terms like allowlist, blocklist, denylist,
          ignorelist, warnlist, safelist, etc. We generally use allowlist and blocklist and prefer consistency where
          possible, but other terms may be more appropriate depending on your use case.

          This is part of a larger initiative to replace exclusionary / harmful language and anti-bias tools and products.
        DOC

        def on_class(node)
          evaluate_node(node)
        end

        private

        def evaluate_node(node)
          return unless should_use_allowlist?(node) || should_use_blocklist?(node)

          add_offense(node)
        end

        def should_use_allowlist?(node)
          node.to_s.downcase.include?('whitelist')
        end

        def should_use_blocklist?(node)
          node.to_s.downcase.include?('blacklist')
        end
      end
    end
  end
end
# rubocop:enable Betterment/AllowlistBlocklist
